/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package gr.abiss.mvn.plugins.jstools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import org.apache.commons.io.IOUtils;
import org.apache.maven.reporting.MavenReportException;
import org.codehaus.plexus.archiver.ArchiverException;
import org.codehaus.plexus.archiver.UnArchiver;
import org.codehaus.plexus.archiver.manager.NoSuchArchiverException;
import org.mozilla.javascript.tools.shell.Main;

/**
 * Goal to generate an unit testing report based on ECMAUnit (<strong>not available yet</strong>)
 * 
 * @goal ecmaunit
 * @version $Id$
 * @author manos
 * 
 */
public class EcmaUnitMojo extends AbstractBaseJstoolsReport {

	/**
	 * A path pointing to a the desired ECMAUnit directory (e.g.
	 * /home/username/stuff/ecmaunit). If no value is provided the
	 * plugin wil use its own internal version.
	 * 
	 * @parameter
	 */
	private String ecmaunitHome;

	/**
	 * Generate an ECMAUnit based unit tests page
	 * @see gr.abiss.mvn.plugins.jstools.AbstractBaseJstoolsReport#doGenerateReport(java.util.Locale)
	 */
	public void doGenerateReport(Locale defaultLocale) throws MavenReportException {
		
	}

	/**
	 * Setup the plugin's internal ECMAUnit version if no other is provided
	 * 
	 * @param defaultLocale
	 * @throws MavenReportException
	 */
	protected void setUp(Locale defaultLocale) throws MavenReportException {
		if (this.ecmaunitHome == null || this.ecmaunitHome.trim().length() == 0) {
			String ecmaunitDistFilename = this.getBundle(defaultLocale).getString(
					"jsdoc.dist.filename");
			this.ecmaunitHome = this.buildDir + "/ecmaunit_run/" + ecmaunitDistFilename;
			String ecmaunitJar = this.baseDir + "/target/" + ecmaunitDistFilename
					+ ".zip";
			// copy the internal jsdoc distribution archive to target/
			try {
				InputStream input = this.getClass().getResourceAsStream(
						"/" + ecmaunitDistFilename + ".zip");
				OutputStream output = new FileOutputStream(ecmaunitJar);
				IOUtils.copy(input, output);
				input.close();
				output.close();
			} catch (FileNotFoundException fe) {
				throw new MavenReportException("Cannot find "
						+ ecmaunitDistFilename + ".zip" + " in classpath", fe);
			} catch (IOException ioe) {
				throw new MavenReportException("Error copying "
						+ ecmaunitDistFilename + ".zip" + " to target dir", ioe);
			}
			// unpack the ecmaunit distribution archive
			File jarFile = new File(ecmaunitJar);
			try {
				UnArchiver unArchiver = this.archiverManager
						.getUnArchiver(jarFile);
				unArchiver.setSourceFile(jarFile);
				File destDir = new File(this.buildDir+"/site").getParentFile();
				destDir.mkdirs();
				unArchiver.setDestDirectory(destDir);
				unArchiver.extract();
			} catch (NoSuchArchiverException ae) {
				throw new MavenReportException("Unknown archiver type", ae);
			} catch (ArchiverException ae) {
				throw new MavenReportException("Error unpacking " + jarFile
						+ ": " + ae.toString(), ae);
			}
		}
	}

	/**
	 * @see org.apache.maven.reporting.AbstractMavenReport#isExternalReport()
	 */
	public boolean isExternalReport() {
		return false;
	}

	/**
	 * @see gr.abiss.mvn.plugins.jstools.AbstractBaseJstoolsReport#getBundleKey()
	 */
	public String getBundleKey() {
		return "ecmaunit";
	}


	/**
	 * Does nothing
	 * @see gr.abiss.mvn.plugins.jstools.AbstractBaseJstoolsReport#tearDown(java.util.Locale)
	 */
	protected void tearDown(Locale defaultLocale) throws MavenReportException {
		// do nothing
		
	}
}
